import sys
import os
import argparse
import logging
import signal
from datetime import datetime

TEMPLATE = '%(asctime)s - %(levelname)s - %(threadName)s [%(thread)d] -  %(message)s'


def valid_date(s):
    try:
        return datetime.strptime(s, "%Y-%m-%d").date()
    except ValueError:
        msg = "Not a valid date: '{0}'.".format(s)
        raise argparse.ArgumentTypeError(msg)


def is_debug_mode():
    log_env = os.getenv('LOG', None)
    if log_env is not None:
        return log_env.upper() == 'DEBUG'
    else:
        return False


def get_logger():
    logger = logging.getLogger('duka')
    if is_debug_mode():
        out_hdlr = logging.StreamHandler(sys.stdout)
        out_hdlr.setFormatter(logging.Formatter(TEMPLATE))
        out_hdlr.setLevel(logging.INFO)
        logger.addHandler(out_hdlr)
        logger.setLevel(logging.INFO)
    return logger


Logger = get_logger()


def set_up_signals():
    def signal_handler(signal, frame):
        sys.exit(0)

    signal.signal(signal.SIGINT, signal_handler)



import csv

from .utils import Logger

TEMPLATE_FILE_NAME = "{}_{}_{:02d}_{:02d}.csv"


def dump(currency, day, ticks):
    file_name = TEMPLATE_FILE_NAME.format(currency, day.year, day.month, day.day)
    Logger.info("Writing {0}".format(file_name))
    with open(file_name, 'w') as csv_file:
        fieldnames = ['time', 'ask', 'bid', 'ask_volume', 'bid_volume']
        writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
        writer.writeheader()
        for tick in ticks:
            writer.writerow(
                {'time': tick[0],
                 'ask': tick[1],
                 'bid': tick[2],
                 'ask_volume': tick[3],
                 'bid_volume': tick[4]})
    Logger.info("{0} completed".format(file_name))



import unittest
from datetime import date
from ..app.app import days


class TestDateGenerator(unittest.TestCase):

    def test_two_dates(self):
        start = date(2016, 1, 20)
        end = date(2016, 1, 21)
        date_list = [d for d in days(start, end)]
        self.assertEqual(len(date_list), 2)
        self.assertEqual(date_list[0], start)
        self.assertEqual(date_list[len(date_list)-1], end)

    def test_one_single_date(self):
        start = date(2016, 1, 20)
        end = date(2016, 1, 20)
        date_list = [d for d in days(start, end)]
        self.assertEqual(len(date_list), 1)
        self.assertEqual(date_list[0], start)
        self.assertEqual(date_list[len(date_list)-1], end)

    def test_skip_saturdays(self):
        start = date(2016, 1, 2)
        end = date(2016, 1, 2)
        date_list = [d for d in days(start, end)]
        self.assertEqual(len(date_list), 0)

    def test_empty_result_when_start_is_bigger_than_end(self):
        start = date(2016, 9, 2)
        end = date(2016, 1, 2)
        date_list = [d for d in days(start, end)]
        self.assertEqual(len(date_list), 0)

    def test_not_fetch_today(self):
        start = date.today()
        end = start
        date_list = [d for d in days(start, end)]
        self.assertEqual(len(date_list), 0)



#!/usr/bin/env python3.5

import argparse
from datetime import date, timedelta

from duka.core import valid_date, set_up_signals
from duka.app import app


def main():
    parser = argparse.ArgumentParser(prog='dukas', usage='%(prog)s [options]')
    parser.add_argument('symbols', metavar='SYMBOLS', type=str, nargs='+', help='symbol list using format EURUSD EURGBP')
    parser.add_argument('-d', '--day', type=valid_date, help='specific day format YYYY-MM-DD (default today)',
                        default=date.today() - timedelta(1))
    parser.add_argument('-s', '--startdate', type=valid_date, help='start date format YYYY-MM-DD (default today)')
    parser.add_argument('-e', '--enddate', type=valid_date, help='end date format YYYY-MM-DD (default today)')
    parser.add_argument('-t', '--thread', type=int, help='number of threads (default 20)', default=5)
    args = parser.parse_args()

    if args.startdate is not None:
        start = args.startdate
    else:
        start = args.day

    if args.enddate is not None:
        end = args.enddate
    else:
        end = args.day

    set_up_signals()
    app(args.symbols, start, end, args.thread)


if __name__ == '__main__':
    main()



from .fetch import fetch_day
from .csv_dumper import dump
from .utils import valid_date, Logger, set_up_signals
from .processor import decompress

__all__ = ['fetch_day', 'decompress', 'dump', 'valid_date', 'Logger', 'set_up_signals']



#!/usr/bin/env python3

from setuptools import setup, find_packages

NAME = "duka"
VERSION = '0.1.2'

setup(
    name=NAME,
    packages=find_packages(),
    install_requires=['aiohttp>=0.21.5'],
    version=VERSION,
    description='Dukascopy Bank SA historical data downloader',
    author='Giuseppe Pes',
    author_email='giuse88@gmail.com',
    url='https://github.com/giuse88/duka',
    download_url='https://github.com/giuse88/duka/tarball/' + VERSION,
    keywords=['dukascopy', 'forex', 'finance', 'historical data', 'price', 'currency'],
    entry_points={
        'console_scripts': [
            'duka = duka.main:main',
        ],
    },
    classifiers=[
        "Environment :: Console",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3 :: Only",
        "Development Status :: 4 - Beta",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: Financial and Insurance Industry",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)




import asyncio
import aiohttp
import threading
import time
from functools import reduce
from io import BytesIO, DEFAULT_BUFFER_SIZE
from ..core.utils import Logger

URL = "https://www.dukascopy.com/datafeed/{currency}/{year}/{month:02d}/{day:02d}/{hour:02d}h_ticks.bi5"

async def get(url):
    buffer = BytesIO()
    id = url[36:].replace('/', " ")
    with aiohttp.ClientSession(loop=asyncio.get_event_loop()) as session:
        start = time.time()
        Logger.info("Fetching {0}".format(id))
        async with session.get(url) as resp:
            while True:
                chunk = await resp.content.read(DEFAULT_BUFFER_SIZE)
                if not chunk:
                    break
                buffer.write(chunk)
    Logger.info("Fetched {0} completed in {1}s".format(id, time.time() - start))
    return buffer.getbuffer()


def fetch_day(symbol, day):

    local_data = threading.local()
    loop = getattr(local_data, 'loop', asyncio.new_event_loop())
    asyncio.set_event_loop(loop)

    url_info = {
        'currency': symbol,
        'year': day.year,
        'month': day.month - 1,
        'day': day.day
    }

    loop = asyncio.get_event_loop()
    tasks = [asyncio.ensure_future(get(URL.format(**url_info, hour=i))) for i in range(24)]
    loop.run_until_complete(asyncio.wait(tasks))

    def add(acc, task):
        acc.write(task.result())
        return acc

    return reduce(add, tasks, BytesIO()).getbuffer()



import lzma
import struct
from datetime import timedelta, datetime

lzma._BUFFER_SIZE = 2048


def decompress_lzma(compressed_buffer):
    return lzma.decompress(compressed_buffer)


def tokenize(buffer):
    token_size = 20
    size = int(len(buffer) / token_size)
    tokens = []
    for i in range(0, size):
        tokens.append(struct.unpack('!IIIff', buffer[i * token_size: (i + 1) * token_size]))
    return tokens


def add_hour(ticks):
    if len(ticks) is 0:
        return ticks

    hour_delta = 0

    if ticks[0][0].weekday() == 6:
        hour_delta = 22

    for index, v in enumerate(ticks):
        if index != 0:
            if ticks[index - 1][0].minute > ticks[index][0].minute:
                hour_delta = ticks[index - 1][0].hour + 1
            else:
                hour_delta = ticks[index - 1][0].hour
        ticks[index] = (v[0] + timedelta(hours=hour_delta), v[1], v[2], v[3], v[4])

    return ticks


def normalize(day, ticks):
    def norm(time, ask, bid, volume_ask, volume_bid):
        date = datetime(day.year, day.month, day.day) + timedelta(milliseconds=time)
        return date, ask / 100000, bid / 100000, round(volume_ask * 1000000), round(volume_bid * 1000000)
    return add_hour(list(map(lambda x: norm(*x), ticks)))


def decompress(day, compressed_buffer):
    if compressed_buffer.nbytes == 0:
        return compressed_buffer
    return normalize(day, tokenize(decompress_lzma(compressed_buffer)))
